"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const CError_1 = __importDefault(require("../errors/CError"));
const buildRegex = (domains) => domains &&
    domains.map((domain) => new RegExp(`^${domain.replace(/\./g, '\\.').replace(/\*/g, '.*')}`));
const domainWhitelistFactory = (pluginOptions) => () => {
    const { whitelist = [] } = pluginOptions || {};
    const validDomains = buildRegex(whitelist);
    return {
        getMiddlewares: () => {
            return [
                // First middleware
                (next) => (req, res) => {
                    if (!validDomains || validDomains.length === 0 || !req.query.image) {
                        return next(req, res);
                    }
                    let valid;
                    try {
                        const url = new URL(req.query.image);
                        valid = validDomains.reduce((acc, test) => acc || test.test(url.host), false);
                    }
                    catch (e) {
                        return next(req, res);
                    }
                    if (!valid) {
                        throw new CError_1.default('The image url is not accessible due to domain restrictions', 'configuration#domains-whitelist', 403);
                    }
                    return next(req, res);
                },
            ];
        },
    };
};
exports.default = domainWhitelistFactory;
