"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const misc_1 = require("../utils/misc");
const micro_1 = require("micro");
const UNKNOWN_ERROR_MESSAGE = 'An unknown error happened :(';
const buildHtmlError = (err, res) => {
    res.setHeader('Content-Type', 'text/html; charset=UTF-8');
    return `
<!doctype html>
<html class="no-js" lang="">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="x-ua-compatible" content="ie=edge">
    <title>Error - Caravaggio</title>
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <style>
      body {
        /*background-color: #4ABDAC;*/
        background: rgb(34,193,195);
        background: linear-gradient(0deg, rgba(34,193,195,1) 0%, rgba(37,194,63,1) 100%);
        margin: 0;
        height: 100vh;
      }
      
      /*Typography*/
      h1 { font-family: 'Droid sans', sans; font-weight: 400; font-style: italic; line-height: 44px;}
      p { font-family: 'Droid Sans', sans-serif; font-size: 15px; font-weight: 400; line-height: 24px; margin: 0 0 14px; }
      a { text-decoration: none; border-bottom: 1px solid #4c3a07; color: inherit; }
      a:hover { color: #fff; background: #4ABDAC; }
      /*/Typography*/

      #container {
        display: flex;
        justify-content: center;
        align-items: center;
        height: 100%;
      }

      #error {
        display:flex; 
        flex-direction: horizontal;
        justify-content: flex-start;
        align-items:center;
        /* background-color: #FF7964;  */
        min-width: 30%; 
        padding: 1em; 
        margin: 0 auto;
        border-radius: 5px;
        color: rgba(255,255,255,0.8);
      }

      #logo {
        margin-right: 1em;
      }
    </style>
  </head>
  <body style="background-color: #4ABDAC;">
    <div id="container">
      <div id="error">
        <div id="logo">
          <img src="//res.cloudinary.com/ramiel/image/upload/v1519252402/caravaggio-logo-error_jvq4fg.png" alt="caravaggio distorted logo"/>
        </div>
        <div>
          <h1>Error</h1>
          <p>${err.message || UNKNOWN_ERROR_MESSAGE}
            <br />See <a target="_blank" href="${(0, misc_1.buildDocumentationLink)(err.docUri)}">${(0, misc_1.buildDocumentationLink)(err.docUri)}</a>
          </p>
        </div>
      </div>
    </div>
  </body>
</html>
`;
};
const buildJsonError = (err, res) => {
    res.setHeader('Content-Type', 'application/json');
    return {
        statusCode: err.statusCode,
        error: err.message || UNKNOWN_ERROR_MESSAGE,
        see: (0, misc_1.buildDocumentationLink)(err.docUri),
    };
};
const buildErrorText = (err, res) => {
    res.setHeader('Content-Type', 'text/plain; charset=UTF-8');
    return `${err.message || UNKNOWN_ERROR_MESSAGE}
  See ${(0, misc_1.buildDocumentationLink)(err.docUri)}`;
};
const errorHandler = (context) => {
    const { config, logger } = context;
    let build;
    switch (config.errors) {
        case 'html':
            build = buildHtmlError;
            break;
        case 'json':
            build = buildJsonError;
            break;
        case 'plain':
        default:
            build = buildErrorText;
            break;
    }
    return (fn) => (req, res) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            return yield fn(req, res);
        }
        catch (err) {
            logger.error(err);
            return (0, micro_1.send)(res, err.statusCode || 500, build(err, res));
        }
    });
};
exports.default = errorHandler;
