"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const blur_1 = __importDefault(require("./blur"));
const o_1 = __importDefault(require("./o"));
const q_1 = __importDefault(require("./q"));
const UnknownOperationError_1 = __importDefault(require("../errors/UnknownOperationError"));
const duotone_1 = __importDefault(require("./duotone"));
const extract_1 = __importDefault(require("./extract"));
const flip_1 = __importDefault(require("./flip"));
const overlay_1 = __importDefault(require("./overlay"));
const progressive_1 = __importDefault(require("./progressive"));
const rotate_1 = __importDefault(require("./rotate"));
const resize_1 = __importDefault(require("./resize"));
const normalizer = (context) => {
    const normalizers = {
        blur: blur_1.default,
        duotone: duotone_1.default,
        ex: extract_1.default,
        extract: extract_1.default,
        flip: flip_1.default,
        o: o_1.default,
        overlay: (0, overlay_1.default)(context),
        progressive: progressive_1.default,
        q: q_1.default,
        resize: resize_1.default,
        rs: resize_1.default,
        rotate: rotate_1.default,
    };
    return {
        normalize: (rawOperations) => {
            const operations = rawOperations.reduce((acc, rawOp) => {
                const normalizer = normalizers[rawOp.operation];
                if (!normalizer) {
                    throw new UnknownOperationError_1.default(rawOp.operation);
                }
                return [...acc, ...normalizer(rawOp)];
            }, []);
            return operations;
        },
    };
};
exports.default = normalizer;
