"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const gravity_1 = require("../utils/gravity");
const cohercer_1 = __importDefault(require("../utils/cohercer"));
const misc_1 = require("../utils/misc");
const overlay = (context) => {
    const { inputImageLoader } = context.pluginManager;
    return ({ url: overlayUrl, g, watermark: wtrmrk, x, y }) => {
        const gravity = (0, gravity_1.getGravityFromParameter)(g);
        const watermark = !!wtrmrk && (0, cohercer_1.default)(wtrmrk).toBool().value();
        const url = (0, cohercer_1.default)(overlayUrl, 'Overlay, url is mandatory', 'overlay.html')
            .toString()
            .value();
        const left = x
            ? (0, cohercer_1.default)(x, 'Extract: the x parameter in in the wrong format.', 'overlay.html')
                .toFloat()
                .value()
            : undefined;
        const top = y
            ? (0, cohercer_1.default)(y, 'Extract: the x parameter in in the wrong format.', 'overlay.html')
                .toFloat()
                .value()
            : undefined;
        return [
            {
                name: 'overlay',
                op: ({ image }) => __awaiter(void 0, void 0, void 0, function* () {
                    let overlay;
                    try {
                        overlay = (yield inputImageLoader(url));
                    }
                    catch (e) {
                        throw new Error(`An error occurred while getting overlay image. ${e.message}`);
                    }
                    const { width: iw, height: ih } = yield image.metadata();
                    const options = {
                        gravity,
                        tile: !!watermark,
                    };
                    if (left !== undefined) {
                        options.left = (0, misc_1.isPercentage)(left)
                            ? (0, misc_1.percentageToPixel)(left, iw)
                            : left;
                    }
                    if (top !== undefined) {
                        options.top = (0, misc_1.isPercentage)(top)
                            ? (0, misc_1.percentageToPixel)(top, ih)
                            : top;
                    }
                    if (options.top !== undefined && !options.left) {
                        options.left = 0;
                    }
                    if (!options.top && options.left !== undefined) {
                        options.top = 0;
                    }
                    return image.composite([Object.assign({ input: overlay }, options)]);
                }),
                params: [
                    {
                        url,
                        x: left,
                        y: top,
                        g: gravity,
                        watermark,
                    },
                ],
            },
        ];
        // return {
        //   transformations: [
        //     {
        //       name: 'overlay',
        //       params: [overlayUrl, ...params],
        //       fn: async (sharp) => {
        //         let overlay;
        //         try {
        //           overlay = await image.get(url);
        //         } catch (e) {
        //           throw new Error(
        //             `An error occurred while getting overlay image. ${e.message}`
        //           );
        //         }
        //         const { width: iw, height: ih } = await sharp.metadata();
        //         const options = {
        //           gravity,
        //           tile: !!watermark,
        //         };
        //         if (left !== undefined) {
        //           options.left = isPercentage(left)
        //             ? percentageToPixel(left, iw)
        //             : left;
        //         }
        //         if (top !== undefined) {
        //           options.top = isPercentage(top)
        //             ? percentageToPixel(top, ih)
        //             : top;
        //         }
        //         if (options.top !== undefined && !options.left) {
        //           options.left = 0;
        //         }
        //         if (!options.top && options.left !== undefined) {
        //           options.top = 0;
        //         }
        //         return sharp.composite([{ input: overlay, ...options }]);
        //       },
        //     },
        //   ],
        // };
    };
};
exports.default = overlay;
