"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const logger_1 = require("../logger");
const DEFAULT_TEMP_DIR = os_1.default.tmpdir();
const filePersistor = ({ basePath = DEFAULT_TEMP_DIR } = { basePath: DEFAULT_TEMP_DIR }) => {
    const logger = (0, logger_1.getMainLogger)();
    const subdir = 'caravaggioCache';
    const baseDir = path_1.default.join(basePath, subdir);
    const getCompleteFilename = (filename) => path_1.default.join(baseDir, filename);
    const cache = {
        flush: () => fs_extra_1.default.emptyDir(baseDir),
        has: (filename) => fs_extra_1.default
            .access(getCompleteFilename(filename), fs_extra_1.default.constants.R_OK)
            .then(() => true)
            .catch(() => false),
        read: (filename) => fs_extra_1.default
            .readFile(getCompleteFilename(filename))
            .then((buffer) => ({
            type: 'buffer',
            data: buffer,
        }))
            .catch((err) => {
            if (err.code === 'ENOENT')
                return null;
            throw err;
        }),
        save: (filename, buffer) => __awaiter(void 0, void 0, void 0, function* () {
            const completeFilename = getCompleteFilename(filename);
            try {
                yield fs_extra_1.default.outputFile(completeFilename, buffer, {
                    // Due to an error in definition of this property
                    // @see https://github.com/DefinitelyTyped/DefinitelyTyped/pull/44637
                    encoding: null,
                });
            }
            catch (e) {
                logger.error(e, `File persistor failed to save file ${completeFilename}`);
            }
            return {
                type: 'buffer',
                data: buffer,
            };
        }),
    };
    return cache;
};
exports.default = filePersistor;
