"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.strictestCacheStrategy = void 0;
const sharp_1 = __importDefault(require("sharp"));
const imageLoader_1 = __importDefault(require("../utils/imageLoader"));
const misc_1 = require("../utils/misc");
const normalizers_1 = __importDefault(require("../normalizers"));
/**
 * Returns the strict-est cache strategy
 */
const strictestCacheStrategy = (operations) => {
    return operations.reduce((acc, op) => {
        const strategy = op.cacheStrategy || 'public';
        if (acc === 'skip')
            return acc;
        return strategy === 'skip'
            ? strategy
            : strategy === 'private'
                ? strategy
                : acc;
    }, 'public');
};
exports.strictestCacheStrategy = strictestCacheStrategy;
const pipelineCreator = (context) => {
    const loader = (0, imageLoader_1.default)(context);
    const logger = context.logger;
    const { normalize } = (0, normalizers_1.default)(context);
    const { config: { defaultOperations = [] }, } = context;
    return ({ url, rawOperations, req }) => __awaiter(void 0, void 0, void 0, function* () {
        const buffer = yield loader.get(url, req);
        const image = (0, sharp_1.default)(buffer);
        const operations = normalize([...defaultOperations, ...rawOperations]);
        const result = yield operations.reduce((acc, { name, op, params }) => __awaiter(void 0, void 0, void 0, function* () {
            if (logger.isLevelEnabled('debug')) {
                logger.debug(`Applying operation "${name}":"${(0, misc_1.stringifyParams)(params)}"`);
            }
            const prevStepImage = yield acc;
            return op({ image: prevStepImage, otherOps: operations, req });
        }), Promise.resolve(image));
        const { data, info } = yield result.toBuffer({ resolveWithObject: true });
        return {
            data,
            format: info.format,
            cacheStrategy: (0, exports.strictestCacheStrategy)(operations),
        };
    });
};
exports.default = pipelineCreator;
