"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PLUGIN_IGNORE_RESULT = void 0;
const flow_1 = require("../utils/flow");
const webImageLoader_1 = __importDefault(require("../basePlugins/webImageLoader"));
const domainWhitelist_1 = __importDefault(require("../basePlugins/domainWhitelist"));
const flatten_1 = __importDefault(require("lodash/flatten"));
exports.PLUGIN_IGNORE_RESULT = Symbol('PLUGIN_IGNORE_RESULT');
const pluginLoader = (config, logger) => {
    let loadedPlugins = [
        {
            name: 'webImageLoader',
            instance: (0, webImageLoader_1.default)()({ config, PLUGIN_IGNORE_RESULT: exports.PLUGIN_IGNORE_RESULT }),
        },
        {
            name: 'domainWhitelist',
            instance: (0, domainWhitelist_1.default)({ whitelist: config.whitelist })({
                config,
                PLUGIN_IGNORE_RESULT: exports.PLUGIN_IGNORE_RESULT,
            }),
        },
    ];
    if (config.plugins) {
        const { paths, plugins = [] } = config.plugins;
        loadedPlugins = plugins
            .filter((plugin) => !plugin.disabled)
            .reduce((acc, { name, instance, options }) => {
            let plugin;
            if (instance) {
                plugin = instance;
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-var-requires
                const loaded = require(require.resolve(name, {
                    paths,
                }));
                plugin = loaded(options);
            }
            return [
                ...acc,
                {
                    name,
                    instance: plugin({
                        config,
                        PLUGIN_IGNORE_RESULT: exports.PLUGIN_IGNORE_RESULT,
                    }),
                },
            ];
        }, loadedPlugins);
    }
    const getFnsFromPlugins = (fn) => loadedPlugins
        .map(({ instance }) => instance[fn])
        .filter((f) => f !== undefined);
    const onPluginError = (e) => {
        logger === null || logger === void 0 ? void 0 : logger.debug(e);
        logger === null || logger === void 0 ? void 0 : logger.debug('Trying next plugin');
    };
    return {
        urlTransform: (url, req) => __awaiter(void 0, void 0, void 0, function* () {
            const fns = getFnsFromPlugins('urlTransform');
            if (fns.length === 0)
                return url;
            try {
                const result = yield (0, flow_1.tryEach)(fns, {
                    onError: onPluginError,
                    ignoreResult: (r) => r === exports.PLUGIN_IGNORE_RESULT,
                })(url, req);
                return result || url;
            }
            catch (e) {
                return url;
            }
        }),
        inputImageLoader: (imageUrl) => __awaiter(void 0, void 0, void 0, function* () {
            const fns = getFnsFromPlugins('inputImageLoader');
            try {
                const result = yield (0, flow_1.tryEach)(fns, {
                    onError: onPluginError,
                    ignoreResult: (r) => r === exports.PLUGIN_IGNORE_RESULT,
                })(imageUrl);
                return result;
            }
            catch (e) {
                throw new Error(`[PLUGIN:inputImageLoader] failed. No plugin is able to download "${imageUrl}"`);
            }
        }),
        getMiddlewares: () => {
            const middlewares = getFnsFromPlugins('getMiddlewares');
            return (0, flatten_1.default)(middlewares.map((m) => m()));
        },
    };
};
exports.default = pluginLoader;
