"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cache_1 = __importDefault(require("../caches/cache"));
const operationParser_1 = __importDefault(require("../utils/operationParser"));
const pipeline_1 = __importDefault(require("../pipeline"));
const sender_1 = __importDefault(require("../utils/sender"));
const CError_1 = __importDefault(require("../errors/CError"));
const defaultOutputCacheConfig = {
    type: 'memory',
    options: {
        limit: 100,
    },
};
const indexRoute = (context) => {
    var _a;
    const { logger, config } = context;
    const cache = (0, cache_1.default)(((_a = config.caches) === null || _a === void 0 ? void 0 : _a.output) || defaultOutputCacheConfig);
    const sender = (0, sender_1.default)(config);
    const pipeline = (0, pipeline_1.default)(context);
    const basePathRegexp = new RegExp(`^${config.basePath}`);
    const basePathReplacer = (url) => config.basePath ? url.replace(basePathRegexp, '') : url;
    const handler = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
        const url = basePathReplacer(req.url || '/');
        const cachedResource = yield cache.get(url);
        if (cachedResource) {
            logger.info(`Cache hit for resource "${url}"`);
            return sender.sendImage(cachedResource, req, res);
        }
        const [path] = url.split('?');
        const rawOperations = (0, operationParser_1.default)(path);
        const imageUrl = req.query.image;
        if (!imageUrl) {
            throw new CError_1.default('You must provide an image url', '', 500);
        }
        logger.debug(imageUrl, 'Image URL');
        logger.debug(rawOperations, 'Raw operations');
        const result = yield pipeline({ url: imageUrl, rawOperations, req });
        yield sender.sendImage(result, req, res, result.cacheStrategy);
        if (result.cacheStrategy === 'public') {
            yield cache.set(url, result);
        }
    });
    return handler;
};
exports.default = indexRoute;
