"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getColorFromParameter = exports.hexToRGB = void 0;
const cohercer_1 = __importDefault(require("./cohercer"));
const COLOR_REGEX = /^([A-Fa-f0-9]{6}|(0[0-9]{2}|1[0-9]{2}|2[0-4][0-9]|25[0-5]){3})(\.\d+)?$/;
/**
 * Convert hex to rgb
 * hex in format 'FFFFFF'
 */
const hexToRGB = (hex) => {
    const value = parseInt(hex, 16);
    const r = value >> 16;
    const g = (value >> 8) & 0xff;
    const b = value & 0xff;
    return { r, g, b };
};
exports.hexToRGB = hexToRGB;
const getColorFromParameter = (param, errorMessage = `Invalid color paramter "${param}".`, docUri = 'resize.html#colors') => {
    const value = (0, cohercer_1.default)(param, errorMessage, docUri)
        .toString()
        .match(COLOR_REGEX)
        .value();
    const dotIndex = value.indexOf('.');
    const number = value.slice(0, dotIndex > 0 ? dotIndex : undefined);
    const alpha = dotIndex > 0 ? value.slice(dotIndex) : '1';
    if (number.length === 6) {
        return Object.assign(Object.assign({}, (0, exports.hexToRGB)(number)), { alpha: parseFloat(alpha) });
    }
    return {
        r: parseInt(number.slice(0, 3), 10),
        g: parseInt(number.slice(3, 6), 10),
        b: parseInt(number.slice(6), 10),
        alpha: parseFloat(alpha),
    };
};
exports.getColorFromParameter = getColorFromParameter;
