"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const zlib_1 = __importDefault(require("zlib"));
const micro_1 = require("micro");
const BROTLI_REGEX = /\bbr\b/;
const DEFLATE_REGEX = /\bdeflate\b/;
const GZIP_REGEX = /\bgzip\b/;
var COMPRESSION_METHODS;
(function (COMPRESSION_METHODS) {
    COMPRESSION_METHODS["br"] = "brotliCompress";
    COMPRESSION_METHODS["deflate"] = "deflate";
    COMPRESSION_METHODS["gzip"] = "gzip";
})(COMPRESSION_METHODS || (COMPRESSION_METHODS = {}));
const compressSend = (method) => (res, code, data) => {
    const promise = new Promise((resolve, reject) => {
        zlib_1.default[method](data, (err, compressedData) => {
            if (err)
                return reject(err);
            return resolve(compressedData);
        });
    });
    return promise.then((compressedData) => {
        res.setHeader('Content-Encoding', method === COMPRESSION_METHODS.br ? 'br' : method);
        (0, micro_1.send)(res, code, compressedData);
    });
};
const compressor = (config) => {
    const sendMethod = config.compress ? compressSend : () => micro_1.send;
    return (req) => {
        const acceptedMethod = req.headers['accept-encoding'];
        const method = false ||
            (BROTLI_REGEX.test(acceptedMethod) && COMPRESSION_METHODS.br) ||
            (DEFLATE_REGEX.test(acceptedMethod) &&
                COMPRESSION_METHODS.deflate) ||
            (GZIP_REGEX.test(acceptedMethod) && COMPRESSION_METHODS.gzip);
        return method ? sendMethod(method) : micro_1.send;
    };
};
exports.default = compressor;
