"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryEach = void 0;
/**
 * Given an array of async functions, return a function that takes the same argument as each function
 * in the array. Each function is executed in series and the first that doesn't crash, returns the
 * value
 * @param fns Array of async functions
 */
const tryEach = (fns, opt) => {
    const { ignoreResult = () => false } = opt || {};
    return (...args) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        for (let i = 0, len = fns.length; i < len; i++) {
            try {
                const pluginresult = yield fns[i](...args);
                if (!ignoreResult(pluginresult)) {
                    return pluginresult;
                }
            }
            catch (e) {
                (_a = opt === null || opt === void 0 ? void 0 : opt.onError) === null || _a === void 0 ? void 0 : _a.call(opt, e);
            }
        }
        throw new Error(`TryEach failed. No function returned a value`);
    });
};
exports.tryEach = tryEach;
