"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getGravityFromParameter = exports.GRAVITY = void 0;
const cohercer_1 = __importDefault(require("./cohercer"));
const GRAVITY_PARAM_REGEX = /^g?(c|n|ne|nw|e|w|s|se|sw|center|centre|north|northeast|northwest|east|west|south|southeast|southwest)$/;
const EXTENDED_GRAVITY_PARAM_REGEX = /^g?(c|n|ne|nw|e|w|s|se|sw|center|centre|north|northeast|northwest|east|west|south|southeast|southwest|auto)$/;
var GRAVITY;
(function (GRAVITY) {
    GRAVITY["c"] = "center";
    GRAVITY["n"] = "north";
    GRAVITY["ne"] = "northeast";
    GRAVITY["nw"] = "northwest";
    GRAVITY["e"] = "east";
    GRAVITY["w"] = "west";
    GRAVITY["s"] = "south";
    GRAVITY["se"] = "southeast";
    GRAVITY["sw"] = "southwest";
    GRAVITY["center"] = "center";
    GRAVITY["centre"] = "centre";
    GRAVITY["north"] = "north";
    GRAVITY["northeast"] = "northeast";
    GRAVITY["northwest"] = "northwest";
    GRAVITY["east"] = "east";
    GRAVITY["west"] = "west";
    GRAVITY["south"] = "south";
    GRAVITY["southeast"] = "southeast";
    GRAVITY["southwest"] = "southwest";
    GRAVITY["auto"] = "attention";
    // With g-prefix
    GRAVITY["gc"] = "center";
    GRAVITY["gn"] = "north";
    GRAVITY["gne"] = "northeast";
    GRAVITY["gnw"] = "northwest";
    GRAVITY["ge"] = "east";
    GRAVITY["gw"] = "west";
    GRAVITY["gs"] = "south";
    GRAVITY["gse"] = "southeast";
    GRAVITY["gsw"] = "southwest";
    GRAVITY["gcenter"] = "center";
    GRAVITY["gcentre"] = "centre";
    GRAVITY["gnorth"] = "north";
    GRAVITY["gnortheast"] = "northeast";
    GRAVITY["gnorthwest"] = "northwest";
    GRAVITY["geast"] = "east";
    GRAVITY["gwest"] = "west";
    GRAVITY["gsouth"] = "south";
    GRAVITY["gsoutheast"] = "southeast";
    GRAVITY["gsouthwest"] = "southwest";
    GRAVITY["gauto"] = "attention";
})(GRAVITY = exports.GRAVITY || (exports.GRAVITY = {}));
const getGravityFromParameter = (gravity, opt) => {
    if (!gravity)
        return;
    const { acceptAuto, error } = Object.assign({ acceptAuto: false, error: `Invalid gravity parameter "${gravity}".` }, opt);
    let value = (0, cohercer_1.default)(gravity, error, 'resize.html#gravity')
        .toString()
        .match(acceptAuto ? EXTENDED_GRAVITY_PARAM_REGEX : GRAVITY_PARAM_REGEX)
        .value();
    value = value[0] === 'g' ? value.slice(1) : value;
    return GRAVITY[value];
};
exports.getGravityFromParameter = getGravityFromParameter;
