"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cache_1 = __importDefault(require("../caches/cache"));
const defaultInputCache = {
    type: 'none',
    options: null,
};
const imageLoader = (context) => {
    var _a;
    const { logger } = context;
    const inputCache = (0, cache_1.default)(((_a = context.config.caches) === null || _a === void 0 ? void 0 : _a.input) || defaultInputCache);
    return {
        get: (url, req) => __awaiter(void 0, void 0, void 0, function* () {
            const cached = yield inputCache.get(url);
            if (cached) {
                logger.debug(`Input cache hit for url "${url}". Do not re-download it.`);
                return cached.data;
            }
            const finalUrl = yield context.pluginManager.urlTransform(url, req);
            const loaded = (yield context.pluginManager.inputImageLoader(finalUrl));
            if (loaded === null) {
                throw new Error(`Cannot load image "${url}"`);
            }
            inputCache.set(url, {
                data: loaded,
            });
            return loaded;
        }),
    };
};
exports.default = imageLoader;
