"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compose = exports.buildDocumentationLink = exports.percentageToPixel = exports.isPercentage = exports.getOutputType = exports.stringifyParams = void 0;
const jsonReplacer = (key, value) => {
    if (Buffer.isBuffer(value)) {
        return `<Buffer of ${value.length} bytes>`;
    }
    if (Array.isArray(value)) {
        return value.map((v) => jsonReplacer(null, v));
    }
    return value;
};
const stringifyParams = (params) => JSON.stringify(params, jsonReplacer, '');
exports.stringifyParams = stringifyParams;
/**
 * Returns the desired output.
 * If an `o` option is specified is used. If it's not specified, or it's `original`,
 * the metadata are checked in the sharp pipeline
 */
const getOutputType = (sharp, operations) => __awaiter(void 0, void 0, void 0, function* () {
    const outputOp = operations.find((op) => {
        return op.name === 'o';
    });
    if (!outputOp || outputOp.params[0] === 'original') {
        return (yield sharp.metadata()).format;
    }
    return outputOp.params[0];
});
exports.getOutputType = getOutputType;
const isPercentage = (percentage) => `${percentage}`.indexOf('.') !== -1;
exports.isPercentage = isPercentage;
const percentageToPixel = (percentage, dimension) => Math.round(percentage * dimension);
exports.percentageToPixel = percentageToPixel;
const buildDocumentationLink = (doc, stripExt = true) => doc
    ? `https://caravaggio.ramielcreations.com/docs/${stripExt ? doc.replace('.html', '') : doc}`
    : 'https://caravaggio.ramielcreations.com/docs';
exports.buildDocumentationLink = buildDocumentationLink;
// eslint-disable-next-line @typescript-eslint/ban-types
const compose = (...fns) => fns.reduce((f, g) => (...args) => f(g(...args)));
exports.compose = compose;
