"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Transform an url into a series of operations following the rules described
 * [here](https://gitlab.com/ramiel/caravaggio/-/issues/29)
 * @param path URL pathname to transform into operations
 * @param opt Define delimiter characters
 */
const operationParser = (path = '', opt) => {
    const { OPERATION_DELIMITER = '/', KEY_VALUE_DELIMITER = ':', OPTION_DELIMITER = ',', } = opt || {};
    return path
        .split(OPERATION_DELIMITER)
        .filter((p) => p !== '')
        .map((opDesc) => {
        const [op, ...options] = opDesc.split(OPTION_DELIMITER);
        if (options.length === 0) {
            const [operation, value] = op.split(KEY_VALUE_DELIMITER);
            return {
                operation,
                value: value !== undefined ? decodeURIComponent(value) : 'true',
            };
        }
        if (op.indexOf(KEY_VALUE_DELIMITER) !== -1) {
            throw new Error(`"${op}" is invalid`);
        }
        return options.reduce((acc, opt) => {
            const [key, value] = opt.split(KEY_VALUE_DELIMITER);
            if (key === 'operation') {
                throw new Error('An operation cannot have a property called "operation"');
            }
            return Object.assign(Object.assign({}, acc), { [key]: value !== undefined ? decodeURIComponent(value) : 'true' });
        }, {
            operation: op,
        });
    });
};
exports.default = operationParser;
