"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sharp_1 = __importDefault(require("sharp"));
const compressor_1 = __importDefault(require("./compressor"));
const senderCreator = (config) => {
    const { browserCache } = config;
    const sendFactory = (0, compressor_1.default)(config);
    const getCacheControlHeaders = (strategy) => {
        switch (strategy) {
            case 'public':
                return typeof browserCache === 'string'
                    ? browserCache
                    : browserCache && browserCache.maxAge
                        ? `max-age=${browserCache.maxAge}`
                        : false;
            case 'private':
                return typeof browserCache === 'string'
                    ? false
                    : browserCache && browserCache.maxAge
                        ? `private, max-age=${browserCache.maxAge}`
                        : false;
            case 'skip':
            default:
                return false;
        }
    };
    const getTypeByMetadata = (buffer) => __awaiter(void 0, void 0, void 0, function* () {
        return (0, sharp_1.default)(buffer)
            .metadata()
            .then(({ format }) => format);
    });
    const getMimeType = (resource) => __awaiter(void 0, void 0, void 0, function* () {
        if (resource.format) {
            return `image/${resource.format}`;
        }
        return `image/${yield getTypeByMetadata(resource.data)}`;
    });
    return {
        sendImage: (resource, req, res, cacheStrategy = 'public') => __awaiter(void 0, void 0, void 0, function* () {
            const send = sendFactory(req);
            switch (resource.type || 'buffer') {
                case 'buffer': {
                    const cacheHeaders = getCacheControlHeaders(cacheStrategy);
                    if (cacheHeaders) {
                        res.setHeader('Cache-Control', cacheHeaders);
                    }
                    res.setHeader('Content-Type', yield getMimeType(resource));
                    return send(res, 200, resource.data);
                }
                // case 'location':
                //   return redirect(res, 301, resource.location);
                default:
                    throw new Error(`Invalid type of resource ${resource.type}`);
            }
        }),
    };
};
exports.default = senderCreator;
