"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sizeToPixel = exports.sizePercentageToPixel = void 0;
const sizePercentageToPixel = (opt, image) => __awaiter(void 0, void 0, void 0, function* () {
    const { width, height } = opt;
    const metadata = yield image.metadata();
    const w = width &&
        (width < 1 ? Math.round(width * metadata.width) : width);
    const h = height &&
        (height < 1 ? Math.round(height * metadata.height) : height);
    return { width: w, height: h };
});
exports.sizePercentageToPixel = sizePercentageToPixel;
/**
 * Given a string representing a size, returns it in numeric values
 * @param size A string repesenting a size. e.g 120x130  0.5x0.9  120x  x130
 * @param image The image to get the size
 */
const sizeToPixel = (size, image) => __awaiter(void 0, void 0, void 0, function* () {
    let width = null;
    let height = null;
    if (size.indexOf(':') !== -1) {
        throw new Error('Aspect ratio not implemented yet');
    }
    else {
        const values = size.split('x');
        width = parseFloat(values[0]) || null;
        height = parseFloat(values[1]) || null;
        const isPercentage = (width && width < 1) || (height && height < 1);
        if (isPercentage) {
            ({ width, height } = yield (0, exports.sizePercentageToPixel)({ width, height }, image));
        }
        else {
            width = width && Math.round(width);
            height = height && Math.round(height);
        }
    }
    return [width, height];
});
exports.sizeToPixel = sizeToPixel;
